function [ boolean ] = arrival_of_profiles( p_nemo_data, p_config )
%ARRIVAL_OF_PROFILES Plots details about iridium data.
%
%   Returns:
%   boolean: False if something went wrong during the plotting process.
%
%   Parameters:
%   p_nemo_data:      Struct, containing all float data (load with nemo.load.data()).
%   p_config:         Global configuration file. Load using nemo.load.config().

%% Initialize return variable
boolean = false;

%% Parameter check
parameter_error = false;

if (~iscell(p_nemo_data) || ~isstruct(p_config))
    parameter_error = true;
end

if (parameter_error) % display error message
    disp([ mfilename ': Wrong parameter! Please check your input parameters carefully!']);
    return;
else % do some cleanup
    clear parameter_error;
end

%% Setup directories
directory = fullfile(p_config.GENERAL.storage_directory_plots, p_config.PLOT_ARRIVAL_OF_PROFILES.storage_directory_image);
directory_fig = fullfile(p_config.GENERAL.storage_directory_plots, p_config.PLOT_ARRIVAL_OF_PROFILES.storage_directory_fig);
image_type = p_config.PLOT_ARRIVAL_OF_PROFILES.image_type;
if ~exist(directory, 'dir')
    system(['mkdir ' directory]);
end
if ~exist(directory_fig, 'dir')
    mkdir(directory_fig);
end

%% Process every float

float_count = length(p_nemo_data);

for o_float = 1:float_count
    
    % collect required data
    
    % count of profiles that have not been send earlier
    unsent_profiles_count = nemo.float.collect( ...
        p_nemo_data{o_float}, ...
        'PROFILE_TECHNICAL_DATA', ...
        'xmit_older_profiles_not_send' ...
    );
    
    % average time when the profile has been transmitted
    iridium_mean_times = nemo.float.collect( ...
        p_nemo_data{o_float}, ...
        'IRIDIUM_DATA', ...
        '' ...
    );
    % extract only datevec from it
    iridium_mean_times = datetime(iridium_mean_times(:, 4:9));
    
    % data about every iridium message that has been transmitted for this profile
    iridium_data = nemo.float.collect( ...
        p_nemo_data{o_float}, ...
        'IRIDIUM_POSITIONS', ...
        '' ...
    );
    iridium_message_count = zeros(1, length(iridium_data));
    % time will be stored in seconds using etime
    iridium_transmittion_time = zeros(1, length(iridium_data));
    for o_tmp = 1:length(iridium_data)
        iridium_message_count(o_tmp) = size(iridium_data{o_tmp}, 1);
        % get iridium transmittion time if available
        if iridium_message_count(o_tmp) > 1
            tmp_min = iridium_data{o_tmp}(1, 4:9);
            tmp_max = iridium_data{o_tmp}(end, 4:9);
            iridium_transmittion_time(o_tmp) = etime(tmp_max, tmp_min);
        else
            iridium_transmittion_time(o_tmp) = NaN;
        end
    end
    
    clear o_tmp iridium_data
    
    % get numbers of the profiles that have been received
    transmitted_profiles = nemo.float.collect(p_nemo_data{o_float}, ...
        'PROFILE_TECHNICAL_DATA', ...
        'xmit_profile_number' ...
    );

    % get gps data
    gps_datetime = nemo.float.collect(p_nemo_data{o_float}, ...
        'SURFACE_GPS_DATA', ...
        '' ...
    );

    best_guess_datevec = nemo.float.collect(p_nemo_data{o_float}, ...
        'SECOND_ORDER_INFORMATION', ...
        'best_guess_datevec' ...
        );
    best_guess_datenum = datenum(best_guess_datevec);

    if isempty(gps_datetime)
       disp(['No arrival of profiles generated for nemo ' ...
            num2str(p_nemo_data{o_float}{1}.FLOAT_IDENTIFICATION.float_serial_number) ...
            '. No GPS information available!']);
        continue;
    end
    
    
    % get plot details
    width = p_config.PLOT_ARRIVAL_OF_PROFILES.image_width;
    height = p_config.PLOT_ARRIVAL_OF_PROFILES.image_height;
    paperunit = p_config.PLOT_ARRIVAL_OF_PROFILES.image_paperunit;
    
    % plot collected data
    
    figure_id = figure('Visible', 'off'); % create figure
    % initialize figure
    set(figure_id,'InvertHardcopy','on');
    set(figure_id,'PaperUnits', paperunit);
    papersize = get(figure_id, 'PaperSize');
    left = (papersize(1)- width)/2;
    bottom = (papersize(2)- height)/2;
    myfiguresize = [left, bottom, width, height];
    set(figure_id,'PaperPosition', myfiguresize);
    
    clear height width left bottom papersize myfiguresize
    
    %set(gcf, 'Position', get(0,'ScreenSize'), 'color', [1 1 1]);
    sub = {};
    
    sub{1} = subplot(4,1,1);
    hold on
    set(gca, 'YLimMode', 'manual', 'YLim', [1, length(transmitted_profiles)+1]);
%     set(gca, 'XLimMode', 'manual', 'XLim', [min_datenum, max_datenum]);
%     set_x_axis();

    disp(['   Processing arrival information of float number ',num2str(p_nemo_data{o_float}{1}.FLOAT_IDENTIFICATION.float_serial_number)])  % OB

    plot(best_guess_datenum, transmitted_profiles,'marker','.','color',[0.5 0.5 0.5],'linestyle','none','linewidth',1,'markersize',20);
    
    % OB START: for float 173 iridium_mean_times is of type datetime, which does not
    % comply with plot command
    % Old Code:
    % plot(iridium_mean_times, transmitted_profiles,'marker','.','color',[0 0 0],'linestyle','none','linewidth',1,'markersize',20);
    
    % New Code
    if isa(iridium_mean_times,'double')
        plot(iridium_mean_times, transmitted_profiles,'marker','.','color',[0 0 0],'linestyle','none','linewidth',1,'markersize',20);
    elseif isa(iridium_mean_times,'datetime')
        plot(datenum(iridium_mean_times), transmitted_profiles,'marker','.','color',[0 0 0],'linestyle','none','linewidth',1,'markersize',20);
    end
    % OB END
    
    t = title([ 'Float ' ...
          num2str(p_nemo_data{o_float}{1}.FLOAT_IDENTIFICATION.float_serial_number) ...
          ' @ ' ...
          datestr(datetime('now', 'Format', 'dd-MMM-yyyy HH:mm:SS')) ...
        ]);
    set(t, 'FontSize', 18);
    clear t
    ylabel('Profile number');
    grid on
    box on
    hold off
    
    % unsent profiles_count
    sub{2} = subplot(4,1,2);
    hold on
%     set(gca, 'XLimMode', 'manual', 'XLim', [min_datenum, max_datenum]);
%     set_x_axis();
    plot(best_guess_datenum, unsent_profiles_count, 'marker','s','color',[0 0 0],'markerfacecolor',[0 0 0],'linestyle','none','markersize',4);
    
    
    ylabel('Unsent profile count');
    grid on
    box on
    hold off
    
    sub{3} = subplot(4,1,3);
    hold on
    plot(best_guess_datenum, iridium_transmittion_time./60, '.-','Markersize',15 );
    ylabel('Transmission time [min]');
    
    grid on
    box on
    hold off
    
    sub{4} = subplot(4,1,4);
    hold on
    plot(best_guess_datenum, iridium_message_count, '.-', 'linestyle', 'none','Markersize',15 );
    
    ylabel('Message count');
    grid on
    box on
    
    hold off
    
    sub = cellfun(@set_x_axis, sub, 'UniformOutput', false);
    
    % store plot
    
    image_filename = [p_config.PLOT_ARRIVAL_OF_PROFILES.image_filename ...
        num2str(p_nemo_data{o_float}{1}.FLOAT_IDENTIFICATION.float_serial_number)];
    figure_filename = [p_config.PLOT_ARRIVAL_OF_PROFILES.figure_filename ...
        num2str(p_nemo_data{o_float}{1}.FLOAT_IDENTIFICATION.float_serial_number)];
    
    print(figure_id, [directory image_filename], image_type);
     
     if (p_config.PLOT_ARRIVAL_OF_PROFILES.save_figure)
         set(gcf,'CreateFcn','set(gcf,''Visible'',''on'')');
         saveas(figure_id, ...
             [directory_fig figure_filename]);
     end
    
    close(figure_id);

end

boolean = true;


    function current_plot = set_x_axis(current_plot)
        
        % get min and max datevec
        min_datenum = min(datenum(best_guess_datevec));
        max_datenum = max(datenum(best_guess_datevec));
        if (min_datenum == max_datenum)
            max_datenum = max_datenum + 1;
        end
        diff_datenum = max_datenum - min_datenum;
        if (diff_datenum > 365)
            x_tick = [min_datenum:365:max_datenum];
        elseif (diff_datenum > 30)
            x_tick = [min_datenum:30:max_datenum];
        elseif (diff_datenum > 5)
            x_tick = [min_datenum:5:max_datenum];
        else
            x_tick = [min_datenum:1:max_datenum];
        end
        if (x_tick(end) < max_datenum)
            x_tick = [ x_tick max_datenum ];
        end
        x_tick_labels = cellstr(datestr(x_tick, 'yyyy-mm-dd'));
        x_tick_labels_top = cellstr(datestr(x_tick, 'yyyy-mm-dd'));
        
        if (~isempty(x_tick_labels))
            x_tick_labels{end} = '';
        end
        
        set(current_plot, 'XLimMode', 'manual', ...
            'XLim', [min_datenum, max_datenum], ...
            'XTick', x_tick, ...
            'XTickLabel', x_tick_labels);
        
    end

end

